# (c) Copyright 2007-2016 Synapse Wireless, Inc.


import sys
import os

from snappy import SnapConstants
from snappy import hooks


def exec_file(filename, globals):
    """Run the builtin execfile our way"""
    if os.path.isfile(filename):
        execfile(filename, globals)
    else:
        # Add \npass\n to fix Python issue with having a comment on the last line, see ticket #1632
        exec "%s\npass\n" % (filename.replace('\r\n', '\n')) in globals


def clearCache(newMods, globals):
    """
    Clean up module cache droppings.
    """
    # print newMods
    globals['_sysCacheRemoved'] = {}
    for name, mod in newMods:
        try:
            # Remove from system modules cache but add to a dict to keep a reference
            # See this for more info: http://mail.python.org/pipermail/python-dev/2007-January/070690.html
            # Essentially it's only safe to delete the module once snappy gen is done otherwise deleting the module
            # when a function's func_globals references something in the module creates problems when GC happens
            globals['_sysCacheRemoved'][name] = sys.modules[name]
            del sys.modules[name]
        except:
            pass

        # Delete .pyc file if found
        try:
            if mod.__file__[-4:] == '.pyc':
                os.remove(mod.__file__)
            elif mod.__file__[-3:] == '.py':
                os.remove(mod.__file__ + 'c')
        except:
            pass


def execFileNoCache(filename, globals):
    """
    Run execfile, cleaning up module cache droppings.
    Returns the set of additional dependencies this file required.
    """
    curMods = set(sys.modules.items())
    # print curMods

    # This is to get around having to modify pyinstaller since it caches filenames in directories
    # REMOVED. Obsoleted by new pyinstaller
    # if hasattr(sys, 'frozen') and hasattr(Globleton, 'userFilesPath') and not sys.platform.startswith('darwin'):
    #  import iu
    #  for dirname in iu._os_listdir.func_defaults[0].keys():
    #    if dirname.startswith(Globleton.userFilesPath):
    #      #print "REMOVING: ", dirname
    #      iu._os_listdir.func_defaults[0].pop(dirname, None)

    try:
        exec_file(filename, globals)
    finally:
        newMods = set(sys.modules.items()) - curMods
        clearCache(newMods, globals)

    return newMods
